#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>

#define SPI_LSB_FIRST
#include "spi_values.h"

#define PS_PORT (PORTB)
#define PS_DDR (DDRB)
#define PS_PIN (PINB)

/* DATA: controller->console (MISO) */
#define PS_DATA (6)
/* CMD: console->controller (MOSI) */
#define PS_CMD (5)
/* ATT: console->controller (/SS) */
#define PS_ATT (4)
/* ATT: console->controller (SCK) */
#define PS_CLK (7)
/* ACK: controller->console */
#define PS_ACK (3)

/* Pedal goes between ground and PD5  */
#define PEDAL_PORT (PORTD)
#define PEDAL_DDR (DDRD)
#define PEDAL_PIN (PIND)
#define PEDAL (5)

/* LED goes between ground and PD4  */
#define LED_PORT (PORTD)
#define LED_DDR (DDRD)
#define LED_PIN (PIND)
#define LED (4)

volatile uint8_t pedal_released_mask = 0;
volatile uint8_t pedal_value = 0;
volatile uint8_t pedal_pressed = 0;

#define PEDAL_THRESHOLD (64)

// The two data bytes sent when the pedal is pressed.
// Change these to change the button sent.
#define PEDAL_PRESSED_DATA_0 (SPI_0xFF)
#define PEDAL_PRESSED_DATA_1 (SPI_0xBF)

volatile enum transfer_state {
	TRANSFER_STATE_DISCONNECTED,
	TRANSFER_STATE_IDLE,
	TRANSFER_STATE_AWAITING_COMMAND,
	TRANSFER_STATE_SENDING_DATA,
	TRANSFER_STATE_FINISHING,
} state;

volatile uint8_t command;
volatile uint8_t data_to_send;


ISR(USI_OVERFLOW_vect) {
	
	uint8_t data = USIDR;
	uint8_t ack = !0;
	
	USISR = _BV(USIOIF);
	
	USIDR = SPI_0xFF;
	
	switch (state) {
		case TRANSFER_STATE_IDLE:
			switch (data) {
				case SPI_0x01:
					state = TRANSFER_STATE_AWAITING_COMMAND;
					USIDR = SPI_0x41; // Digital pad.
					break;
				default:
					state = TRANSFER_STATE_DISCONNECTED;
					ack = 0;
					break;
			}
			break;
		case TRANSFER_STATE_AWAITING_COMMAND:
			command = data;
			switch (command) {
				case SPI_0x42:
					state = TRANSFER_STATE_SENDING_DATA;
					data_to_send = 2; // 2 bytes of data.
					pedal_pressed = (pedal_value >= PEDAL_THRESHOLD); // Determine whether the pedal is pressed or not.
					USIDR = SPI_0x5A; // Data coming.
					break;
				default:
					state = TRANSFER_STATE_IDLE;
					ack = 0;
					break;
			}
			break;
		case TRANSFER_STATE_SENDING_DATA:
			switch (2 - data_to_send) {
				case 0:
					if (pedal_pressed) {
						data = PEDAL_PRESSED_DATA_0;
					} else {
						data = SPI_0xFF;
					}
					USIDR = data;
					break;
				case 1:
					if (pedal_pressed) {
						data = PEDAL_PRESSED_DATA_1;
					} else {
						data = SPI_0xFF;
					}
					USIDR = data;
					break;
			}
			if (--data_to_send == 0) {
				state = TRANSFER_STATE_FINISHING;
			}
			break;
		case TRANSFER_STATE_FINISHING:
			state = TRANSFER_STATE_IDLE;
			ack = 0;
			break;
		case TRANSFER_STATE_DISCONNECTED:
			ack = 0;
			break;
	}
	
	if (ack) {
		PS_DDR |= _BV(PS_ACK);
		_delay_us(1);
		PS_DDR &= ~_BV(PS_ACK);
	} else if (state == TRANSFER_STATE_DISCONNECTED) {
		PS_DDR &= ~_BV(PS_DATA);
		USICR &= ~_BV(USIOIE);
	}
	
}

void pedal_init(void) {
	
	uint8_t old_pedal_released_mask;
	uint8_t pedal_released_mask_stable_count;
	
	PEDAL_PORT |= _BV(PEDAL);
	
	pedal_released_mask_stable_count = 0;
	for (; pedal_released_mask_stable_count < 200; ++pedal_released_mask_stable_count) {
		_delay_us(10);
		old_pedal_released_mask = pedal_released_mask;
		pedal_released_mask = PEDAL_PIN & _BV(PEDAL);
		if (pedal_released_mask != old_pedal_released_mask) {
			pedal_released_mask_stable_count = 0;
		}
	}
	
	
	
	
}


int main(void) {
	
	pedal_init();
	
	LED_DDR |= _BV(LED);
	
	PS_PORT |= _BV(PS_ATT); // Weak pull-up on ATT (/SS)
	USICR = _BV(USIWM0) | _BV(USICS1); // USI in three-wire mode with external clock source.
	
	sei();
	
	for (;;) {
		
		if ((PEDAL_PIN & _BV(PEDAL)) == pedal_released_mask) {
			if (pedal_value > 0) --pedal_value;
		} else {
			if (pedal_value < (PEDAL_THRESHOLD * 2) - 1) ++pedal_value;
		}
		
		if (pedal_value < PEDAL_THRESHOLD) {
			LED_PORT &= ~_BV(LED);
		} else {
			LED_PORT |= _BV(LED);
		}
		
		if (PS_PIN & _BV(PS_ATT)) {
			// Slave is not selected.
			PS_DDR &= ~_BV(PS_DATA);
			USICR &= ~_BV(USIOIE);
			USISR = _BV(USIOIF);
			USIDR = 0xFF;
			state = TRANSFER_STATE_IDLE;
		} else if (state != TRANSFER_STATE_DISCONNECTED) {
			// Slave is selected.
			PS_DDR |= _BV(PS_DATA);
			USICR |= _BV(USIOIE);
		}
	}
}